#line 1 "rustlexer.cpp"

#line 3 "rustlexer.cpp"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer rust__create_buffer
#define yy_delete_buffer rust__delete_buffer
#define yy_scan_buffer rust__scan_buffer
#define yy_scan_string rust__scan_string
#define yy_scan_bytes rust__scan_bytes
#define yy_init_buffer rust__init_buffer
#define yy_flush_buffer rust__flush_buffer
#define yy_load_buffer_state rust__load_buffer_state
#define yy_switch_to_buffer rust__switch_to_buffer
#define yypush_buffer_state rust_push_buffer_state
#define yypop_buffer_state rust_pop_buffer_state
#define yyensure_buffer_stack rust_ensure_buffer_stack
#define yy_flex_debug rust__flex_debug
#define yyin rust_in
#define yyleng rust_leng
#define yylex rust_lex
#define yylineno rust_lineno
#define yyout rust_out
#define yyrestart rust_restart
#define yytext rust_text
#define yywrap rust_wrap
#define yyalloc rust_alloc
#define yyrealloc rust_realloc
#define yyfree rust_free

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 4
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

#ifdef yy_create_buffer
#define rust__create_buffer_ALREADY_DEFINED
#else
#define yy_create_buffer rust__create_buffer
#endif

#ifdef yy_delete_buffer
#define rust__delete_buffer_ALREADY_DEFINED
#else
#define yy_delete_buffer rust__delete_buffer
#endif

#ifdef yy_scan_buffer
#define rust__scan_buffer_ALREADY_DEFINED
#else
#define yy_scan_buffer rust__scan_buffer
#endif

#ifdef yy_scan_string
#define rust__scan_string_ALREADY_DEFINED
#else
#define yy_scan_string rust__scan_string
#endif

#ifdef yy_scan_bytes
#define rust__scan_bytes_ALREADY_DEFINED
#else
#define yy_scan_bytes rust__scan_bytes
#endif

#ifdef yy_init_buffer
#define rust__init_buffer_ALREADY_DEFINED
#else
#define yy_init_buffer rust__init_buffer
#endif

#ifdef yy_flush_buffer
#define rust__flush_buffer_ALREADY_DEFINED
#else
#define yy_flush_buffer rust__flush_buffer
#endif

#ifdef yy_load_buffer_state
#define rust__load_buffer_state_ALREADY_DEFINED
#else
#define yy_load_buffer_state rust__load_buffer_state
#endif

#ifdef yy_switch_to_buffer
#define rust__switch_to_buffer_ALREADY_DEFINED
#else
#define yy_switch_to_buffer rust__switch_to_buffer
#endif

#ifdef yypush_buffer_state
#define rust_push_buffer_state_ALREADY_DEFINED
#else
#define yypush_buffer_state rust_push_buffer_state
#endif

#ifdef yypop_buffer_state
#define rust_pop_buffer_state_ALREADY_DEFINED
#else
#define yypop_buffer_state rust_pop_buffer_state
#endif

#ifdef yyensure_buffer_stack
#define rust_ensure_buffer_stack_ALREADY_DEFINED
#else
#define yyensure_buffer_stack rust_ensure_buffer_stack
#endif

#ifdef yylex
#define rust_lex_ALREADY_DEFINED
#else
#define yylex rust_lex
#endif

#ifdef yyrestart
#define rust_restart_ALREADY_DEFINED
#else
#define yyrestart rust_restart
#endif

#ifdef yylex_init
#define rust_lex_init_ALREADY_DEFINED
#else
#define yylex_init rust_lex_init
#endif

#ifdef yylex_init_extra
#define rust_lex_init_extra_ALREADY_DEFINED
#else
#define yylex_init_extra rust_lex_init_extra
#endif

#ifdef yylex_destroy
#define rust_lex_destroy_ALREADY_DEFINED
#else
#define yylex_destroy rust_lex_destroy
#endif

#ifdef yyget_debug
#define rust_get_debug_ALREADY_DEFINED
#else
#define yyget_debug rust_get_debug
#endif

#ifdef yyset_debug
#define rust_set_debug_ALREADY_DEFINED
#else
#define yyset_debug rust_set_debug
#endif

#ifdef yyget_extra
#define rust_get_extra_ALREADY_DEFINED
#else
#define yyget_extra rust_get_extra
#endif

#ifdef yyset_extra
#define rust_set_extra_ALREADY_DEFINED
#else
#define yyset_extra rust_set_extra
#endif

#ifdef yyget_in
#define rust_get_in_ALREADY_DEFINED
#else
#define yyget_in rust_get_in
#endif

#ifdef yyset_in
#define rust_set_in_ALREADY_DEFINED
#else
#define yyset_in rust_set_in
#endif

#ifdef yyget_out
#define rust_get_out_ALREADY_DEFINED
#else
#define yyget_out rust_get_out
#endif

#ifdef yyset_out
#define rust_set_out_ALREADY_DEFINED
#else
#define yyset_out rust_set_out
#endif

#ifdef yyget_leng
#define rust_get_leng_ALREADY_DEFINED
#else
#define yyget_leng rust_get_leng
#endif

#ifdef yyget_text
#define rust_get_text_ALREADY_DEFINED
#else
#define yyget_text rust_get_text
#endif

#ifdef yyget_lineno
#define rust_get_lineno_ALREADY_DEFINED
#else
#define yyget_lineno rust_get_lineno
#endif

#ifdef yyset_lineno
#define rust_set_lineno_ALREADY_DEFINED
#else
#define yyset_lineno rust_set_lineno
#endif

#ifdef yywrap
#define rust_wrap_ALREADY_DEFINED
#else
#define yywrap rust_wrap
#endif

#ifdef yyalloc
#define rust_alloc_ALREADY_DEFINED
#else
#define yyalloc rust_alloc
#endif

#ifdef yyrealloc
#define rust_realloc_ALREADY_DEFINED
#else
#define yyrealloc rust_realloc
#endif

#ifdef yyfree
#define rust_free_ALREADY_DEFINED
#else
#define yyfree rust_free
#endif

#ifdef yytext
#define rust_text_ALREADY_DEFINED
#else
#define yytext rust_text
#endif

#ifdef yyleng
#define rust_leng_ALREADY_DEFINED
#else
#define yyleng rust_leng
#endif

#ifdef yyin
#define rust_in_ALREADY_DEFINED
#else
#define yyin rust_in
#endif

#ifdef yyout
#define rust_out_ALREADY_DEFINED
#else
#define yyout rust_out
#endif

#ifdef yy_flex_debug
#define rust__flex_debug_ALREADY_DEFINED
#else
#define yy_flex_debug rust__flex_debug
#endif

#ifdef yylineno
#define rust_lineno_ALREADY_DEFINED
#else
#define yylineno rust_lineno
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* begin standard C++ headers. */

/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define YY_SC_TO_UI(c) ((YY_CHAR) (c))

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START
/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin  )
#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

extern int yyleng;

extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )
#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = NULL; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = NULL;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart ( FILE *input_file  );
void yy_switch_to_buffer ( YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE yy_create_buffer ( FILE *file, int size  );
void yy_delete_buffer ( YY_BUFFER_STATE b  );
void yy_flush_buffer ( YY_BUFFER_STATE b  );
void yypush_buffer_state ( YY_BUFFER_STATE new_buffer  );
void yypop_buffer_state ( void );

static void yyensure_buffer_stack ( void );
static void yy_load_buffer_state ( void );
static void yy_init_buffer ( YY_BUFFER_STATE b, FILE *file  );
#define YY_FLUSH_BUFFER yy_flush_buffer( YY_CURRENT_BUFFER )

YY_BUFFER_STATE yy_scan_buffer ( char *base, yy_size_t size  );
YY_BUFFER_STATE yy_scan_string ( const char *yy_str  );
YY_BUFFER_STATE yy_scan_bytes ( const char *bytes, int len  );

void *yyalloc ( yy_size_t  );
void *yyrealloc ( void *, yy_size_t  );
void yyfree ( void *  );

#define yy_new_buffer yy_create_buffer
#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}
#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}
#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define rust_wrap() (/*CONSTCOND*/1)
#define YY_SKIP_YYWRAP
typedef flex_uint8_t YY_CHAR;

FILE *yyin = NULL, *yyout = NULL;

typedef int yy_state_type;

extern int yylineno;
int yylineno = 1;

extern char *yytext;
#ifdef yytext_ptr
#undef yytext_ptr
#endif
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state ( void );
static yy_state_type yy_try_NUL_trans ( yy_state_type current_state  );
static int yy_get_next_buffer ( void );
static void yynoreturn yy_fatal_error ( const char* msg  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;
#define YY_NUM_RULES 130
#define YY_END_OF_BUFFER 131
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static const flex_int16_t yy_accept[486] =
    {   0,
        0,    0,  101,  101,  113,  113,  131,  129,  127,  124,
      126,  112,  129,  129,  129,  118,  118,  128,  128,  128,
      128,  128,  128,  128,  128,  128,  128,  128,  128,  128,
      128,  128,  118,  128,  128,  128,  128,  128,  128,  128,
      128,  128,  128,  128,  128,  128,  128,  128,  128,  128,
      128,  128,  101,  103,  104,  102,  113,  114,  115,  117,
      130,  125,    0,    0,    0,    0,  100,  107,    0,  118,
        0,    0,    0,    0,  128,  128,  128,  128,  128,  128,
      128,  128,  128,  128,   31,  128,   24,  128,  128,  128,
      128,  128,  128,  128,  128,  128,  128,  128,  128,  128,

      128,  128,  128,  110,  118,  128,  118,  128,  128,  128,
      128,   53,  128,  128,  128,  128,  128,  128,   60,  128,
      128,  128,  128,  128,  128,  128,   66,  128,  128,  128,
      128,   11,   68,  128,   70,  128,  128,  128,  128,  128,
      128,  128,  128,  128,  128,  128,  128,  128,  128,  128,
      128,  128,  128,  128,  128,    5,  128,  128,  128,  128,
      128,  101,  105,  102,  102,  106,  113,    0,  116,    0,
      111,  122,    0,  122,  107,  118,    0,  118,  119,  120,
      121,  128,  128,   26,  128,  128,  128,  128,  128,  128,
      128,  128,  128,  128,  128,  128,  128,  128,  128,   32,

      128,  128,  128,  128,  128,  128,  128,  128,  128,  128,
      128,   46,  128,  118,  118,  118,  128,  128,  128,  128,
       55,  128,  128,  128,  128,  128,  128,  128,    9,   10,
      128,  128,   67,   12,   13,   14,  128,  128,   71,  128,
      128,  128,  109,  128,   76,  128,  128,  128,  128,   81,
      128,   83,  128,  128,  128,  128,   15,  128,  128,  128,
      128,    6,    7,    8,  128,  108,  128,  128,  128,  128,
      128,  123,    0,  128,  128,  128,   17,  128,  128,  128,
      128,   23,  128,  128,  128,   36,   35,  128,   50,  128,
      128,  128,   16,   20,  128,  128,   49,  128,   22,  128,

      128,  128,  128,  128,    1,  128,    4,  128,  128,  128,
       61,   62,  128,  128,  128,   69,  128,   72,  128,  128,
       75,  128,  128,   79,   80,   82,  128,   85,  128,  128,
      128,  128,  128,   91,   92,  128,  128,  128,  128,  128,
      128,  128,    0,  118,   34,   33,   28,   19,  128,  128,
      128,  128,  128,   25,  128,  128,  128,  128,  128,   21,
      128,  128,  128,  128,  128,  128,  128,   56,   57,  128,
       59,  128,   64,   65,    2,   73,   74,  128,  128,  128,
      128,  128,  128,   89,   90,  128,  128,  128,    3,  128,
       97,   98,   99,  128,  128,  128,  128,   39,  128,  128,

       47,  128,   48,  128,   44,  128,  128,  128,  128,   54,
      128,   63,  128,  128,   84,   86,   87,   88,   93,   94,
      128,  128,   37,   18,  128,  128,  128,  128,  128,  128,
       27,  128,  128,   52,  128,  128,  128,   95,   96,  128,
      128,  128,   38,  128,  128,  128,   45,   51,   58,   77,
       78,  128,  128,  128,   29,  128,  128,  128,  128,  128,
       30,  128,  128,  128,  128,  128,  128,  128,   40,  128,
      128,  128,  128,  128,  128,   43,  128,  128,  128,  128,
      128,   42,  128,   41,    0
    } ;

static const YY_CHAR yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        2,    2,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    5,    6,    7,    1,    1,    1,    8,    1,
        1,    9,   10,    1,   10,   11,   12,   13,   14,   15,
       16,   17,   18,   19,   18,   20,   21,    1,    1,    1,
        1,    1,    1,    1,   22,   23,   24,   25,   26,   27,
       28,   28,   29,   28,   28,   28,   30,   31,   32,   33,
       28,   34,   35,   36,   28,   37,   28,   28,   28,   28,
       38,   39,   40,    1,   41,    1,   42,   43,   44,   45,

       46,   47,   48,   49,   50,   51,   52,   53,   54,   55,
       56,   57,   58,   59,   60,   61,   62,   63,   64,   65,
       66,   67,    1,   68,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const YY_CHAR yy_meta[69] =
    {   0,
        1,    1,    2,    3,    1,    4,    1,    1,    5,    6,
        1,    1,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    8,    8,    8,    8,    9,    8,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,    1,    1,    1,
        7,    8,    8,    8,    8,    9,    8,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,   11
    } ;

static const flex_int16_t yy_base[501] =
    {   0,
        0,    0,   66,   68,   70,   75,  350, 1977, 1977, 1977,
      345, 1977,   75,  294,   73,  104,   40,   73,  121,  165,
       51,   90,   47,   42,   96,   55,  105,  112,  110,  141,
       73,  142,  217,  275,  150,  254,  255,  269,  304,  266,
      322,  284,  310,  308,  330,  335,  341,  353,  391,  365,
      296,  371,    0, 1977,  328,  119,  286, 1977,  314, 1977,
      300, 1977,  262,  247,  249,  248, 1977,    0,  422,    0,
      245,  160,  439,    0,    0,  114,  170,  152,  136,  158,
      373,  160,  406,  171,  246,  408,  344,  374,  414,  420,
      392,  418,  432,  425,  430,  436,  440,  444,  445,  449,

      456,  471,  460, 1977,  517,  576,    0,  486,  608,  497,
      499,  501,  512,  517,  521,  545,  523,  559,  556,  560,
      588,  592,  214,  208,  587,  603,  611,  614,  200,  197,
      182,    0,  612,  623,  617,  632,  626,  635,  646,  651,
      655,  650,  659,  670,  674,  675,  678,  686,  698,  699,
      701,  707,  171,  161,  158,    0,  702,  722,  726,  727,
      728,    0, 1977,  505,  506, 1977,  132,  153, 1977,  113,
      104, 1977,  124,  119,    0,  613,  776,  785,  291,  794,
        0,  580,  495,  466,  470,  762,  581,  558,  551,  771,
      763,  777,  776,  778,  515,  783,  785,  796,  795,  790,

      794,  800,  807,  812,  813,  814,  818,  822,  829,  823,
      827,  828,  879,  106,  102,   89,  723,  858,  859,  861,
      864,  880,  882,  883,  885,  904,  905,  908,    0,    0,
      896,  910,  919,    0,    0,    0,  923,  924,  928,  932,
      937,  950,  952,  955,  956,  961,  964,  975,  976,  979,
      984,  988,  990, 1002, 1008, 1003, 1014, 1026, 1027, 1032,
     1038,    0,    0,    0, 1040, 1046, 1051, 1052, 1064, 1066,
     1067, 1977,   74,  852, 1069,  881,  876, 1080, 1076, 1078,
     1084, 1086, 1088, 1090, 1094, 1095, 1101, 1110, 1100, 1105,
     1109, 1111, 1115, 1116, 1125, 1127, 1121, 1129, 1123, 1133,

     1134, 1148, 1149, 1152, 1157, 1168, 1170, 1171, 1176, 1191,
     1182, 1190, 1195, 1203, 1204, 1209, 1215, 1217, 1223, 1229,
     1228, 1241, 1237, 1242, 1250, 1255, 1261, 1256, 1269, 1274,
     1275, 1280, 1283, 1288, 1289, 1303, 1294, 1309, 1314, 1322,
     1327, 1336, 1384, 1393, 1141, 1143, 1162, 1330, 1332, 1373,
     1338, 1337, 1371, 1339, 1367, 1377, 1366, 1389, 1379, 1383,
     1393, 1395, 1391, 1400, 1412, 1414, 1415, 1409, 1420, 1423,
     1428, 1434, 1439, 1442, 1443, 1447, 1448, 1461, 1463, 1472,
     1477, 1466, 1475, 1486, 1488, 1491, 1497, 1499, 1510, 1512,
     1513, 1515, 1518, 1521, 1523, 1534, 1530, 1511, 1535, 1539,

     1540, 1541, 1545, 1546, 1547, 1559, 1552, 1566, 1570, 1568,
     1571, 1579, 1582, 1584, 1593, 1597, 1598, 1602, 1606, 1611,
     1620, 1617, 1616, 1622, 1626, 1630, 1631, 1632, 1642, 1646,
     1636, 1652, 1650, 1653, 1661, 1669, 1672, 1666, 1674, 1675,
     1677, 1682, 1683, 1684, 1688, 1699, 1692, 1703, 1706, 1707,
     1708, 1716, 1724, 1717, 1701, 1726, 1732, 1730, 1728, 1734,
     1738, 1739, 1764, 1750, 1745, 1771, 1746, 1752, 1756, 1757,
     1770, 1772, 1776, 1762, 1777, 1778, 1793, 1783, 1787, 1788,
     1789, 1797, 1798, 1799, 1977, 1857, 1868, 1879, 1884, 1894,
     1905, 1916, 1927, 1938, 1949, 1955, 1956, 1959, 1963, 1969

    } ;

static const flex_int16_t yy_def[501] =
    {   0,
      485,    1,  486,  486,  487,  487,  485,  485,  485,  485,
      485,  485,  485,  488,  485,  485,   16,  489,  489,  489,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,  485,   33,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,  490,  485,  485,  491,  492,  485,  485,  485,
      492,  485,  485,  493,  485,  494,  485,  495,  485,   17,
      496,  485,  485,  497,  498,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,

       20,   20,   20,  485,  498,  489,   33,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,  498,  498,   34,   34,   34,   34,  498,  498,
      498,  498,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,  498,  498,  498,  498,   34,   34,   34,   34,
       34,  490,  485,  491,  491,  485,  492,  492,  485,  493,
      493,  485,  485,  485,  495,  499,  485,  485,  485,  485,
      497,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,

       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,  498,  213,  106,  109,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,  498,  498,
       34,   34,   34,  498,  498,  498,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,  498,  498,  498,   34,   34,   34,   34,   34,   34,
       34,  485,  500,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,

       20,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,  485,  485,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   20,   20,   20,   20,   20,   20,   20,

       20,   20,   20,   20,   20,   20,   20,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   34,   34,   34,   34,   34,   34,   34,   20,
       20,   20,   20,   20,   20,   20,   20,   34,   34,   34,
       34,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   20,   20,   20,
       20,   20,   20,   20,    0,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485

    } ;

static const flex_int16_t yy_nxt[2046] =
    {   0,
        8,    9,   10,   11,    8,   12,   13,   14,    8,    8,
        8,   15,   16,   17,   17,   17,   17,   17,   17,   17,
       17,   18,   19,   20,   21,   22,   23,   24,   25,   24,
       26,   27,   28,   29,   30,   31,   32,    8,    8,    8,
       33,   34,   35,   36,   37,   38,   39,   40,   40,   41,
       40,   40,   42,   43,   40,   44,   45,   40,   46,   47,
       48,   49,   50,   51,   40,   52,   40,    8,   54,   55,
       54,   55,   58,   59,   56,   60,   56,   58,   59,   63,
       60,   67,  485,  343,   68,   75,   75,   75,   75,   75,
       75,   75,   75,   75,   76,  485,   81,   76,  485,   76,

       82,   87,   76,   76,  485,   88,   83,   76,   61,   84,
       91,  485,   64,   61,   69,  485,   70,   70,   70,   70,
       70,   70,   70,   70,   70,   76,  272,  165,  102,   71,
      166,  272,   77,   75,   75,   75,   75,   75,   75,   75,
       75,   75,   76,  171,   70,   76,   72,   85,   76,   71,
       89,   76,  171,   94,   86,   95,   90,   76,  167,   73,
       76,   92,   76,   93,   76,   76,   76,   76,   74,   76,
      168,   71,  179,  179,  264,  263,   78,   75,   75,   75,
       75,   75,   75,   75,   75,   75,   96,  103,   76,  262,
       97,  185,  108,   98,   76,  113,   99,   76,  236,  182,

      179,  100,  108,  183,   76,  114,  101,   76,  115,  108,
       76,  235,   76,   76,  186,   76,  184,   79,  234,  189,
       80,  104,   76,   76,  230,   76,  191,   69,  229,  105,
      105,  105,  105,  105,  105,  105,  105,  105,   76,   76,
       76,   76,  106,   76,   76,   76,   76,   76,   76,   76,
       76,   76,   76,   76,  177,  174,  172,  107,  108,  108,
      108,  108,  109,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,   71,  485,  171,   75,   75,   75,
       75,   75,   75,   75,   75,   75,  108,  108,   76,   64,

       76,   76,  116,  179,  179,  167,  108,  108,  108,  117,
      119,  108,  118,  108,  108,  108,  169,  110,  108,  123,
      108,  120,  124,  121,  168,  108,  108,  111,  108,  137,
      163,  179,   66,  122,  112,  129,  108,  130,  108,  138,
      131,  132,  485,  108,  160,  125,  108,   62,  108,  485,
      108,  139,  108,  126,  142,  108,  108,  485,  127,  128,
      108,  485,  108,  108,  108,  140,  485,  108,  133,  108,
      143,  141,  108,  194,  108,  134,  135,  108,  485,  485,
      146,  136,  108,  108,  485,  485,  147,  108,  144,  108,
      148,  145,  485,  108,  108,  108,   76,  485,  485,   76,

      108,  149,  150,  485,  153,  108,  154,  108,  485,  155,
      156,  151,  108,  108,  159,  187,  485,  108,  152,  188,
      161,  485,  485,  108,  108,   76,   76,  485,   76,  195,
      108,  485,  485,  108,  176,  176,  176,  176,  176,  176,
      176,  176,  176,  108,   76,  157,  198,   76,  485,  192,
      158,  180,  180,  180,  180,  180,  180,  180,   76,  485,
       76,   76,  176,   76,  485,  197,   76,  190,  193,   76,
       76,  485,   76,   76,  196,   76,  200,   76,  199,  180,
       76,  485,   76,  201,   76,   76,  485,   76,  203,  202,
      204,   76,   76,  206,  485,   76,   76,   76,  207,   76,

       76,   76,  210,  212,   76,  211,  205,  208,   76,  485,
      209,   76,   76,  485,  165,   76,  485,  166,   76,  485,
      485,   76,   76,   76,  276,   76,   76,   69,  108,  105,
      105,  105,  105,  105,  105,  105,  105,  105,  108,  108,
      275,  108,  213,  108,  485,  108,  485,   76,  218,  108,
       76,  108,  485,  108,  108,  219,  217,  105,  108,  108,
      108,  485,  213,  108,  108,  108,  222,   76,  286,  108,
       76,  108,  220,  108,  485,  108,  108,  224,  485,  485,
      108,  221,  108,  485,   71,  177,  223,  108,  214,  214,
      214,  214,  214,  214,  214,  214,  214,  108,  108,  279,

      225,  108,  108,   76,  108,  485,   76,  280,  108,  485,
       76,  108,  108,   76,  485,  108,  215,  177,  108,  226,
      214,  214,  214,  214,  214,  214,  214,  214,  214,  108,
      108,  485,   76,   76,  108,   76,   76,  485,  273,  231,
      108,  274,  278,  485,  108,  108,  108,  108,  216,  227,
      108,  108,  228,  108,  108,  108,  108,  232,  273,  108,
      108,  485,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  233,  108,  108,  108,  108,  108,  108,  237,
      273,  238,  108,  485,  108,  108,  239,  108,  108,  241,
      240,  108,  108,  108,  108,  243,  246,  108,  108,  485,

      485,  108,  108,  108,  247,  108,  242,  108,  485,  108,
      108,  108,  108,  244,  108,  245,  250,  108,  108,  248,
      108,  252,  108,  485,  485,  249,  108,  108,  108,  108,
      254,  485,  251,  108,  108,  253,  485,  108,  108,  256,
      108,  108,  259,  108,  108,  108,  485,  485,  485,  108,
      108,  108,  255,  108,  108,  258,  257,  108,  108,  108,
      108,  265,  260,  261,  108,  108,  108,  266,  108,  108,
      108,  267,  269,  271,  108,  108,  270,  485,  108,  108,
      108,  108,  108,  302,  268,  108,  108,  108,  178,  178,
      178,  178,  178,  178,  178,  178,  178,  178,  178,  178,

      178,  178,  178,  178,  178,  178,  180,  180,  180,  180,
      180,  180,  180,  281,   76,   76,  178,   76,   76,  282,
      283,  284,  485,   76,  485,  178,   76,  277,   76,   76,
       76,   76,   76,   76,  180,   76,  485,   76,  287,  285,
       76,  289,   76,  288,  290,   76,   76,   76,   76,   76,
       76,   76,   76,  293,  291,   76,  294,  296,  295,   76,
      485,  292,   76,  297,   76,   76,   76,   76,   76,   76,
       76,  298,  299,   76,   76,   76,  485,   76,   76,   76,
       76,   76,   76,   76,   76,  485,  300,  301,  177,  485,
      485,  214,  214,  214,  214,  214,  214,  214,  214,  214,

      108,  108,  485,  108,   76,  303,  108,   76,  485,  485,
      108,  108,  345,  305,  304,  485,  108,  108,  108,  214,
      108,  306,  108,  108,  108,  108,  347,  108,   76,  485,
      485,   76,  108,   76,  108,  108,   76,  108,  108,  108,
      307,  108,  308,  309,  108,  310,  108,  108,  108,  311,
      108,  315,  108,  313,  485,  314,  108,  108,  312,  485,
      108,  108,  108,  108,  108,  108,  108,  108,  485,  108,
      108,  108,  485,  485,  108,  316,  108,  485,  108,  108,
      108,  485,  108,  108,  108,  485,  485,  108,  318,  108,
      317,  108,  108,  320,  108,  319,  108,  108,  108,  485,

      321,  485,  108,  108,  108,  485,  108,  108,  108,  108,
      485,  108,  485,  108,  108,  108,  108,  108,  108,  325,
      322,  108,  323,  108,  485,  485,  108,  108,  108,  326,
      108,  108,  108,  485,  108,  108,  108,  324,  108,  485,
      108,  485,  108,  108,  108,  108,  485,  108,  328,  108,
      108,  327,  485,  329,  108,  108,  108,  485,  485,  485,
      108,  108,  108,  330,  485,  485,  108,  108,  108,  108,
      485,  332,  485,  108,  108,  331,  333,  334,  108,  108,
      108,  336,  108,  335,  108,  108,  108,  485,  108,  337,
      108,  108,  108,  108,  108,  485,  485,  108,  108,  108,

      485,  485,  485,  108,  108,  108,  108,  485,  108,  108,
      108,  108,  339,  485,  485,  346,  108,  338,  341,  342,
      485,   76,  340,  108,   76,  108,  108,  348,   76,  355,
      350,   76,   76,   76,  485,   76,  351,  349,   76,   76,
       76,   76,   76,   76,  353,   76,   76,   76,  352,   76,
       76,  356,   76,   76,  354,   76,   76,   76,  358,  485,
      357,   76,   76,  359,   76,   76,   76,   76,   76,  360,
       76,   76,  361,   76,  485,   76,   76,   76,   76,   76,
       76,   76,   76,  362,   76,   76,   76,  363,   76,   76,
      108,  108,  364,   76,  108,   76,   76,  485,   76,  108,

      108,  108,  485,  366,  108,  367,  365,  108,  108,  108,
      108,  108,  108,  108,   76,  485,  108,   76,  108,  368,
      108,  485,  108,  108,  108,  370,  485,  108,  108,  108,
      108,  369,  108,  108,  108,  108,  371,  108,  485,  485,
      485,  108,  108,  108,  485,  108,  108,  108,  373,  108,
      108,  108,  485,  372,  108,  108,  374,  108,  485,  108,
      375,  108,  108,  108,  485,  108,  485,  108,  108,  108,
      108,  108,  485,  485,  108,  108,  108,  377,  376,  108,
      108,  108,  108,  108,  108,  485,  378,  108,  108,  108,
      485,  485,  108,  108,  108,  379,  108,  108,  108,  485,

      108,  108,  108,  108,  485,  485,  485,  108,  108,  108,
      485,  108,  485,  108,  108,  108,  108,  108,  383,  380,
      108,  108,  108,  382,  381,  108,  108,  108,  108,  485,
      108,  108,  108,  108,  108,  108,  108,  485,  384,  108,
      108,  108,  108,  385,  386,  108,  108,  108,  108,  387,
      485,  108,  485,  108,  389,  108,  108,  485,  485,  485,
      388,  108,  108,  485,  108,  485,  108,  391,  108,  108,
      485,  397,  392,  108,  108,  390,  485,  485,  108,  108,
      393,  108,   76,  396,  394,   76,  108,   76,  108,   76,
       76,   76,   76,   76,   76,  108,  344,  344,  344,  344,

      344,  344,  344,  344,  344,  344,  344,  344,  344,  344,
      344,  344,  344,  344,  395,  398,  404,  485,   76,   76,
      401,   76,   76,   76,  344,   76,   76,  399,   76,   76,
      402,   76,   76,  344,   76,   76,  406,  400,   76,  403,
      485,   76,  405,   76,   76,   76,   76,   76,   76,  407,
       76,  108,   76,  408,  108,   76,  108,  108,  485,  485,
      410,  108,  108,  485,  108,  108,  108,  108,  108,  409,
      108,  108,  108,  108,  108,  108,  108,  411,  485,  108,
      108,  108,  108,  485,  108,  108,  108,  108,  412,  108,
      108,  108,  485,  108,  108,  108,  485,  485,  108,  108,

      108,  108,  108,  108,  485,  108,  108,  108,  108,  417,
      485,  485,  414,  108,  108,  108,  485,  108,  108,  108,
      108,  413,  108,  416,  108,  108,  415,  108,  108,  108,
      108,  108,  485,  108,  108,  418,  108,  419,  108,  108,
      108,  108,  420,  108,  421,  108,  485,  108,  485,  108,
      108,  108,  108,  422,  108,  108,  108,  108,  108,  425,
      108,  485,  108,   76,  108,  108,   76,  108,  485,  108,
      108,  108,  108,   76,  108,   76,   76,  108,   76,  426,
      427,  423,   76,  485,  485,   76,   76,   76,  424,   76,
       76,   76,   76,  429,  428,   76,   76,   76,   76,   76,

       76,  430,   76,  431,   76,  485,  432,   76,  108,  433,
      108,   76,  108,  108,   76,  485,  434,  485,  108,  485,
      108,  108,  108,  108,  108,  108,  108,  108,  437,  108,
      108,  108,  435,  485,  108,  108,  108,  436,  108,  108,
      108,  108,  485,  108,  108,  108,  485,  485,  108,  108,
      108,  485,  108,  108,  108,  485,  108,  108,  108,  108,
      485,  108,  108,  108,  438,  108,  485,  444,   76,  439,
      108,   76,  108,  445,   76,  485,  108,   76,   76,  108,
      440,   76,   76,   76,   76,   76,   76,   76,   76,  442,
      443,   76,  108,  485,   76,  108,  441,   76,   76,  447,

      446,   76,  108,  108,   76,  108,  449,   76,  108,  108,
      448,  108,  108,  108,  108,  450,  108,  451,  108,  452,
      108,  108,  453,  454,  108,  108,  108,   76,  108,   76,
       76,  108,   76,  108,   76,   76,   76,   76,   76,   76,
       76,  455,  457,   76,   76,  108,  456,   76,  108,  108,
      108,   76,  459,   76,   76,  108,   76,  485,  108,  108,
      108,  458,  108,  485,  485,  108,  108,  108,   76,   76,
      461,   76,   76,  462,  463,  485,   76,  460,   76,   76,
       76,   76,   76,   76,   76,   76,   76,   76,  464,  465,
       76,   76,  467,   76,   76,  468,  470,   76,   76,  466,

       76,   76,   76,  469,   76,   76,  471,   76,   76,   76,
      472,   76,   76,  475,   76,  474,   76,   76,  485,   76,
      477,  473,   76,   76,   76,   76,   76,   76,   76,   76,
       76,   76,   76,   76,  479,   76,  476,  478,  480,   76,
       76,   76,   76,   76,  483,   76,  482,  481,   76,   76,
       76,   76,   76,   76,   76,  485,  484,   53,   53,   53,
       53,   53,   53,   53,   53,   53,   53,   53,   57,   57,
       57,   57,   57,   57,   57,   57,   57,   57,   57,   65,
      485,   65,   65,   65,   65,   65,   65,   65,   65,   65,
       76,   76,   76,   76,  162,  485,  485,  162,  485,  162,

      162,  162,  162,  162,  162,  164,  485,  485,  164,  164,
      164,  164,  164,  164,  164,  164,  167,  485,  485,  485,
      167,  167,  167,  167,  167,  167,  167,  170,  485,  170,
      170,  170,  170,  170,  170,  170,  170,  170,  173,  485,
      173,  173,  173,  173,  173,  173,  173,  173,  173,  175,
      485,  485,  175,  175,  175,  175,  175,  175,  175,  175,
      178,  178,  181,  181,  181,   75,   75,   75,   75,  176,
      485,  176,  485,  176,  344,  344,    7,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,

      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485
    } ;

static const flex_int16_t yy_chk[2046] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    3,    3,
        4,    4,    5,    5,    3,    5,    4,    6,    6,   13,
        6,   15,   17,  273,   15,   18,   18,   18,   18,   18,
       18,   18,   18,   18,   24,   17,   21,   24,  216,   23,

       21,   23,   23,   21,   17,   23,   21,   26,    5,   21,
       26,  215,   13,    6,   16,  214,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   31,  174,   56,   31,   16,
       56,  173,   18,   19,   19,   19,   19,   19,   19,   19,
       19,   19,   22,  171,   16,   22,   16,   22,   25,   16,
       25,   25,  170,   28,   22,   29,   25,   27,  168,   16,
       27,   27,   29,   27,   28,   29,   76,   28,   16,   76,
      167,   16,   72,   72,  155,  154,   19,   20,   20,   20,
       20,   20,   20,   20,   20,   20,   30,   32,   79,  153,
       30,   79,   35,   30,   32,   35,   30,   32,  131,   77,

       72,   30,   35,   77,   78,   35,   30,   78,   35,   35,
       80,  130,   82,   80,   80,   82,   78,   20,  129,   82,
       20,   33,   77,   84,  124,   77,   84,   33,  123,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   33,   33,   71,   66,   65,   33,   33,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   33,   33,   33,   34,   64,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   36,   37,   85,   63,

       34,   85,   36,  179,  179,   61,   36,   37,   40,   36,
       37,   38,   36,   36,   37,   34,   59,   34,   40,   39,
       34,   38,   39,   38,   57,   40,   42,   34,   38,   42,
       55,  179,   14,   38,   34,   41,   42,   41,   51,   42,
       41,   41,   34,   42,   51,   39,   39,   11,   51,    7,
       44,   43,   43,   39,   44,   51,   39,    0,   39,   39,
       44,    0,   43,   39,   41,   43,    0,   44,   41,   43,
       44,   43,   45,   87,   41,   41,   41,   46,    0,    0,
       46,   41,   45,   47,    0,    0,   47,   46,   45,   45,
       47,   45,    0,   47,   46,   48,   87,    0,    0,   87,

       47,   47,   47,    0,   49,   48,   49,   50,    0,   49,
       49,   48,   48,   52,   50,   81,    0,   50,   48,   81,
       52,    0,    0,   52,   50,   81,   88,    0,   81,   88,
       52,    0,    0,   49,   69,   69,   69,   69,   69,   69,
       69,   69,   69,   49,   91,   49,   91,   91,    0,   86,
       49,   73,   73,   73,   73,   73,   73,   73,   83,    0,
       86,   83,   69,   86,    0,   90,   89,   83,   86,   89,
       92,    0,   90,   92,   89,   90,   93,   94,   92,   73,
       94,    0,   95,   94,   93,   95,    0,   93,   96,   95,
       96,   96,   97,   98,    0,   97,   98,   99,   99,   98,

       99,  100,  102,  103,  100,  102,   97,  100,  101,    0,
      101,  101,  103,  164,  165,  103,  164,  165,  184,    0,
        0,  184,  185,  102,  185,  185,  102,  105,  108,  105,
      105,  105,  105,  105,  105,  105,  105,  105,  108,  110,
      183,  111,  105,  112,    0,  108,    0,  183,  111,  110,
      183,  111,    0,  112,  113,  113,  110,  105,  111,  114,
      112,    0,  105,  115,  113,  117,  115,  195,  195,  114,
      195,  113,  114,  115,    0,  117,  114,  117,    0,    0,
      115,  114,  117,    0,  105,  106,  116,  116,  106,  106,
      106,  106,  106,  106,  106,  106,  106,  116,  119,  188,

      118,  118,  120,  189,  116,    0,  189,  189,  119,    0,
      188,  118,  120,  188,    0,  119,  106,  109,  118,  120,
      109,  109,  109,  109,  109,  109,  109,  109,  109,  125,
      121,    0,  182,  187,  122,  182,  187,    0,  176,  125,
      121,  182,  187,    0,  122,  126,  125,  121,  109,  121,
      109,  122,  122,  127,  133,  126,  128,  126,  176,  135,
      109,    0,  126,  127,  133,  134,  128,  109,  137,  135,
      127,  133,  128,  128,  136,  134,  135,  138,  137,  134,
      176,  136,  134,    0,  136,  137,  137,  138,  139,  139,
      138,  136,  142,  140,  138,  140,  142,  141,  139,    0,

        0,  143,  142,  140,  143,  139,  139,  141,    0,  142,
      140,  143,  144,  140,  141,  141,  145,  146,  143,  144,
      147,  146,  144,    0,    0,  144,  145,  146,  148,  144,
      147,    0,  145,  145,  146,  146,    0,  147,  148,  149,
      149,  150,  151,  151,  157,  148,    0,    0,    0,  152,
      149,  150,  148,  151,  157,  150,  149,  149,  150,  152,
      151,  157,  151,  152,  158,  217,  152,  158,  159,  160,
      161,  158,  160,  161,  158,  217,  160,    0,  159,  160,
      161,  158,  217,  217,  159,  159,  160,  161,  177,  177,
      177,  177,  177,  177,  177,  177,  177,  178,  178,  178,

      178,  178,  178,  178,  178,  178,  180,  180,  180,  180,
      180,  180,  180,  190,  186,  191,  177,  186,  191,  191,
      192,  193,    0,  190,    0,  178,  190,  186,  193,  192,
      194,  193,  192,  194,  180,  196,    0,  197,  196,  194,
      197,  198,  200,  197,  199,  200,  201,  199,  198,  201,
      199,  198,  202,  203,  201,  202,  204,  206,  205,  203,
        0,  202,  203,  207,  204,  205,  206,  204,  205,  206,
      207,  208,  209,  207,  208,  210,    0,  208,  210,  211,
      212,  209,  211,  212,  209,    0,  210,  211,  213,    0,
        0,  213,  213,  213,  213,  213,  213,  213,  213,  213,

      218,  219,    0,  220,  274,  218,  221,  274,    0,    0,
      218,  219,  274,  220,  219,    0,  221,  218,  219,  213,
      220,  222,  222,  221,  223,  224,  276,  225,  277,    0,
        0,  277,  222,  276,  223,  224,  276,  225,  231,  222,
      223,  223,  224,  224,  225,  225,  226,  227,  231,  226,
      228,  232,  232,  228,    0,  231,  226,  227,  227,    0,
      228,  233,  232,  226,  227,  237,  238,  228,    0,  232,
      239,  233,    0,    0,  240,  237,  238,    0,  233,  241,
      239,    0,  237,  238,  240,    0,    0,  239,  240,  241,
      238,  240,  242,  242,  243,  241,  241,  244,  245,    0,

      244,    0,  242,  246,  243,    0,  247,  244,  245,  242,
        0,  243,    0,  246,  244,  245,  247,  248,  249,  249,
      246,  250,  247,  247,    0,    0,  251,  248,  249,  251,
      252,  250,  253,    0,  248,  249,  251,  248,  250,    0,
      252,    0,  253,  251,  254,  256,    0,  252,  254,  253,
      255,  253,    0,  255,  254,  256,  257,    0,    0,    0,
      255,  254,  256,  256,    0,    0,  257,  255,  258,  259,
        0,  258,    0,  257,  260,  257,  259,  260,  258,  259,
      261,  265,  265,  261,  260,  258,  259,    0,  266,  265,
      261,  260,  265,  267,  268,    0,    0,  261,  266,  265,

        0,    0,    0,  267,  268,  266,  269,    0,  270,  271,
      267,  268,  268,    0,    0,  275,  269,  267,  270,  271,
        0,  275,  269,  269,  275,  270,  271,  278,  279,  287,
      280,  279,  278,  280,    0,  278,  281,  279,  282,  281,
      283,  282,  284,  283,  284,  284,  285,  286,  283,  285,
      286,  288,  289,  287,  285,  289,  287,  290,  291,    0,
      290,  291,  288,  292,  291,  288,  292,  293,  294,  295,
      293,  294,  296,  297,    0,  299,  297,  295,  299,  296,
      295,  298,  296,  298,  298,  300,  301,  300,  300,  301,
      302,  303,  301,  345,  304,  346,  345,    0,  346,  305,

      302,  303,    0,  303,  304,  304,  302,  302,  303,  305,
      306,  304,  307,  308,  347,    0,  305,  347,  309,  306,
      306,    0,  307,  308,  311,  309,    0,  306,  309,  307,
      308,  308,  312,  310,  311,  309,  310,  313,    0,    0,
        0,  311,  312,  310,    0,  314,  315,  313,  314,  312,
      310,  316,    0,  313,  313,  314,  315,  317,    0,  318,
      317,  316,  314,  315,    0,  319,    0,  317,  316,  318,
      321,  320,    0,    0,  317,  319,  318,  320,  319,  323,
      321,  320,  319,  322,  324,    0,  322,  321,  320,  323,
        0,    0,  325,  322,  324,  323,  323,  326,  328,    0,

      322,  324,  325,  327,    0,    0,    0,  326,  328,  325,
        0,  329,    0,  327,  326,  328,  330,  331,  331,  327,
      327,  329,  332,  330,  329,  333,  330,  331,  329,    0,
      334,  335,  332,  330,  331,  333,  337,    0,  332,  332,
      334,  335,  333,  333,  335,  336,  337,  334,  335,  336,
        0,  338,    0,  337,  338,  336,  339,    0,    0,    0,
      337,  338,  336,    0,  340,    0,  339,  340,  338,  341,
        0,  352,  341,  339,  340,  339,    0,    0,  342,  341,
      342,  340,  348,  351,  349,  348,  341,  349,  342,  352,
      351,  354,  352,  351,  354,  342,  343,  343,  343,  343,

      343,  343,  343,  343,  343,  344,  344,  344,  344,  344,
      344,  344,  344,  344,  350,  353,  361,    0,  357,  355,
      357,  357,  355,  353,  343,  350,  353,  355,  350,  356,
      358,  359,  356,  344,  359,  360,  363,  356,  360,  359,
        0,  358,  362,  363,  358,  361,  363,  362,  361,  364,
      362,  368,  364,  365,  365,  364,  366,  367,    0,    0,
      367,  368,  369,    0,  365,  370,  366,  367,  368,  366,
      371,  365,  369,  366,  367,  370,  372,  370,    0,  369,
      371,  373,  370,    0,  374,  375,  372,  371,  372,  376,
      377,  373,    0,  372,  374,  375,    0,    0,  373,  376,

      377,  374,  375,  378,    0,  379,  376,  377,  382,  382,
        0,    0,  379,  378,  380,  379,    0,  383,  382,  381,
      378,  378,  379,  381,  380,  382,  380,  383,  384,  381,
      385,  380,    0,  386,  383,  383,  381,  386,  384,  387,
      385,  388,  387,  386,  388,  384,    0,  385,    0,  387,
      386,  388,  389,  390,  390,  391,  387,  392,  388,  396,
      393,    0,  389,  398,  390,  391,  398,  392,    0,  389,
      393,  390,  391,  394,  392,  395,  394,  393,  395,  397,
      399,  394,  397,    0,    0,  397,  396,  399,  395,  396,
      399,  400,  401,  402,  400,  401,  402,  403,  404,  405,

      403,  404,  405,  406,  407,    0,  407,  407,  408,  408,
      410,  406,  409,  411,  406,    0,  409,    0,  408,    0,
      410,  412,  409,  411,  413,  408,  414,  410,  414,  409,
      411,  412,  411,    0,  413,  415,  414,  413,  412,  416,
      417,  413,    0,  414,  418,  415,    0,    0,  419,  416,
      417,    0,  415,  420,  418,    0,  416,  417,  419,  422,
        0,  418,  421,  420,  421,  419,    0,  429,  423,  422,
      420,  423,  421,  429,  424,    0,  422,  424,  425,  421,
      425,  425,  426,  427,  428,  426,  427,  428,  431,  427,
      428,  431,  433,    0,  429,  434,  426,  429,  430,  432,

      430,  430,  433,  435,  432,  434,  435,  432,  438,  433,
      433,  436,  434,  435,  437,  436,  439,  437,  438,  440,
      435,  436,  441,  442,  437,  438,  439,  440,  436,  441,
      440,  437,  441,  439,  442,  443,  444,  442,  443,  444,
      445,  444,  446,  445,  447,  448,  445,  447,  449,  450,
      451,  446,  453,  455,  446,  448,  455,    0,  449,  450,
      451,  452,  448,    0,    0,  449,  450,  451,  452,  454,
      456,  452,  454,  457,  458,    0,  453,  454,  456,  453,
      459,  456,  458,  459,  457,  458,  460,  457,  459,  460,
      461,  462,  463,  461,  462,  464,  466,  465,  467,  462,

      465,  467,  464,  465,  468,  464,  467,  468,  469,  470,
      468,  469,  470,  472,  474,  471,  463,  474,    0,  463,
      474,  470,  471,  466,  472,  471,  466,  472,  473,  475,
      476,  473,  475,  476,  477,  478,  473,  475,  478,  479,
      480,  481,  479,  480,  481,  477,  480,  479,  477,  482,
      483,  484,  482,  483,  484,    0,  483,  486,  486,  486,
      486,  486,  486,  486,  486,  486,  486,  486,  487,  487,
      487,  487,  487,  487,  487,  487,  487,  487,  487,  488,
        0,  488,  488,  488,  488,  488,  488,  488,  488,  488,
      489,  489,  489,  489,  490,    0,    0,  490,    0,  490,

      490,  490,  490,  490,  490,  491,    0,    0,  491,  491,
      491,  491,  491,  491,  491,  491,  492,    0,    0,    0,
      492,  492,  492,  492,  492,  492,  492,  493,    0,  493,
      493,  493,  493,  493,  493,  493,  493,  493,  494,    0,
      494,  494,  494,  494,  494,  494,  494,  494,  494,  495,
        0,    0,  495,  495,  495,  495,  495,  495,  495,  495,
      496,  496,  497,  497,  497,  498,  498,  498,  498,  499,
        0,  499,    0,  499,  500,  500,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,

      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int yy_flex_debug;
int yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "rustlexer.lpp"
#define YY_NO_INPUT 1
#line 16 "rustlexer.lpp"

#include <stdio.h>
#include "tokenizer.h"
#line 1322 "rustlexer.cpp"

#line 1324 "rustlexer.cpp"

#define INITIAL 0
#define comment 1
#define string_literal 2

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals ( void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy ( void );

int yyget_debug ( void );

void yyset_debug ( int debug_flag  );

YY_EXTRA_TYPE yyget_extra ( void );

void yyset_extra ( YY_EXTRA_TYPE user_defined  );

FILE *yyget_in ( void );

void yyset_in  ( FILE * _in_str  );

FILE *yyget_out ( void );

void yyset_out  ( FILE * _out_str  );

			int yyget_leng ( void );

char *yyget_text ( void );

int yyget_lineno ( void );

void yyset_lineno ( int _line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap ( void );
#else
extern int yywrap ( void );
#endif
#endif

#ifndef YY_NO_UNPUT
    
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy ( char *, const char *, int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen ( const char * );
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput ( void );
#else
static int input ( void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (yy_size_t) max_size, yyin)) == 0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex (void);

#define YY_DECL int yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer( yyin, YY_BUF_SIZE );
		}

		yy_load_buffer_state(  );
		}

	{
#line 24 "rustlexer.lpp"

#line 1543 "rustlexer.cpp"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)] ;
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 486 )
					yy_c = yy_meta[yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1977 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 25 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 26 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 27 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 28 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 29 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 30 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 31 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 32 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 33 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 34 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 35 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 36 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 37 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 38 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 39 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 40 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 41 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 42 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 43 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 44 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 45 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 46 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 47 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 48 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 49 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 50 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 51 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 52 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 53 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 54 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 55 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 56 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 57 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 58 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 59 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 60 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 61 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 62 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 63 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 64 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 65 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 66 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 67 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 68 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 69 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 70 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 71 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 72 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 73 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 74 "rustlexer.lpp"
{ return(TOKENIZER_TYPE); }
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 76 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 77 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 78 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 79 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 80 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 81 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 82 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 83 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 84 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 85 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 86 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 87 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 88 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 89 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 90 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 91 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 92 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 93 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 94 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 70:
YY_RULE_SETUP
#line 95 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 96 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 97 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 98 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 99 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 100 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 76:
YY_RULE_SETUP
#line 101 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 77:
YY_RULE_SETUP
#line 102 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 103 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 79:
YY_RULE_SETUP
#line 104 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 105 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 81:
YY_RULE_SETUP
#line 106 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 82:
YY_RULE_SETUP
#line 107 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 83:
YY_RULE_SETUP
#line 108 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 84:
YY_RULE_SETUP
#line 109 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 85:
YY_RULE_SETUP
#line 110 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 86:
YY_RULE_SETUP
#line 111 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 87:
YY_RULE_SETUP
#line 112 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 88:
YY_RULE_SETUP
#line 113 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 89:
YY_RULE_SETUP
#line 114 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 90:
YY_RULE_SETUP
#line 115 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 91:
YY_RULE_SETUP
#line 116 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 92:
YY_RULE_SETUP
#line 117 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 93:
YY_RULE_SETUP
#line 118 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 94:
YY_RULE_SETUP
#line 119 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 95:
YY_RULE_SETUP
#line 120 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 96:
YY_RULE_SETUP
#line 121 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 97:
YY_RULE_SETUP
#line 122 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 98:
YY_RULE_SETUP
#line 123 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 99:
YY_RULE_SETUP
#line 124 "rustlexer.lpp"
{ return(TOKENIZER_KEYWORD); }
	YY_BREAK
case 100:
YY_RULE_SETUP
#line 126 "rustlexer.lpp"
{ BEGIN(comment); return(TOKENIZER_COMMENT); }
	YY_BREAK
case 101:
YY_RULE_SETUP
#line 127 "rustlexer.lpp"
{ return(TOKENIZER_COMMENT); }
	YY_BREAK
case 102:
YY_RULE_SETUP
#line 128 "rustlexer.lpp"
{ return(TOKENIZER_COMMENT); }
	YY_BREAK
case 103:
/* rule 103 can match eol */
YY_RULE_SETUP
#line 129 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 104:
YY_RULE_SETUP
#line 130 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 105:
/* rule 105 can match eol */
YY_RULE_SETUP
#line 131 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 106:
YY_RULE_SETUP
#line 132 "rustlexer.lpp"
{ BEGIN(INITIAL); return(TOKENIZER_COMMENT); }
	YY_BREAK
case 107:
YY_RULE_SETUP
#line 134 "rustlexer.lpp"
{ return(TOKENIZER_COMMENT); }
	YY_BREAK
case 108:
YY_RULE_SETUP
#line 136 "rustlexer.lpp"
{ return(TOKENIZER_DIRECTIVE); }
	YY_BREAK
case 109:
YY_RULE_SETUP
#line 137 "rustlexer.lpp"
{ return(TOKENIZER_DIRECTIVE); }
	YY_BREAK
case 110:
YY_RULE_SETUP
#line 138 "rustlexer.lpp"
{ return(TOKENIZER_DIRECTIVE); }
	YY_BREAK
case 111:
YY_RULE_SETUP
#line 139 "rustlexer.lpp"
{ return(TOKENIZER_DIRECTIVE); }
	YY_BREAK
case 112:
YY_RULE_SETUP
#line 141 "rustlexer.lpp"
{ BEGIN(string_literal);    return(TOKENIZER_LITERAL); }
	YY_BREAK
case 113:
YY_RULE_SETUP
#line 142 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); }
	YY_BREAK
case 114:
/* rule 114 can match eol */
YY_RULE_SETUP
#line 143 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 115:
YY_RULE_SETUP
#line 144 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 116:
/* rule 116 can match eol */
YY_RULE_SETUP
#line 145 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 117:
YY_RULE_SETUP
#line 146 "rustlexer.lpp"
{ BEGIN(INITIAL);           return(TOKENIZER_LITERAL); }
	YY_BREAK
case 118:
YY_RULE_SETUP
#line 148 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); } 
	YY_BREAK
case 119:
YY_RULE_SETUP
#line 149 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); }
	YY_BREAK
case 120:
YY_RULE_SETUP
#line 150 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); }
	YY_BREAK
case 121:
YY_RULE_SETUP
#line 151 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); }
	YY_BREAK
case 122:
YY_RULE_SETUP
#line 153 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); }
	YY_BREAK
case 123:
YY_RULE_SETUP
#line 154 "rustlexer.lpp"
{ return(TOKENIZER_LITERAL); }
	YY_BREAK
case 124:
/* rule 124 can match eol */
YY_RULE_SETUP
#line 156 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 125:
/* rule 125 can match eol */
YY_RULE_SETUP
#line 157 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 126:
YY_RULE_SETUP
#line 158 "rustlexer.lpp"
{ return(TOKENIZER_NEWLINE); }
	YY_BREAK
case 127:
YY_RULE_SETUP
#line 159 "rustlexer.lpp"
{ return(TOKENIZER_TEXT);    }
	YY_BREAK
case 128:
YY_RULE_SETUP
#line 160 "rustlexer.lpp"
{ return(TOKENIZER_TEXT);    }
	YY_BREAK
case 129:
YY_RULE_SETUP
#line 161 "rustlexer.lpp"
{ return(TOKENIZER_TEXT);    }
	YY_BREAK
case 130:
YY_RULE_SETUP
#line 163 "rustlexer.lpp"
ECHO;
	YY_BREAK
#line 2256 "rustlexer.cpp"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(comment):
case YY_STATE_EOF(string_literal):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( yywrap(  ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = (yytext_ptr);
	int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr) - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc( (void *) b->yy_ch_buf,
							 (yy_size_t) (b->yy_buf_size + 2)  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = NULL;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if (((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc(
			(void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, (yy_size_t) new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		YY_CURRENT_BUFFER_LVALUE->yy_buf_size = (int) (new_size - 2);
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	yy_state_type yy_current_state;
	char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 486 )
				yy_c = yy_meta[yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	int yy_is_jam;
    	char *yy_cp = (yy_c_buf_p);

	YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 486 )
			yy_c = yy_meta[yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
	yy_is_jam = (yy_current_state == 485);

		return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_UNPUT

#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (int) ((yy_c_buf_p) - (yytext_ptr));
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap(  ) )
						return 0;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer( yyin, YY_BUF_SIZE );
	}

	yy_init_buffer( YY_CURRENT_BUFFER, input_file );
	yy_load_buffer_state(  );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state(  );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc( (yy_size_t) (b->yy_buf_size + 2)  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * 
 */
    void yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree( (void *) b->yy_ch_buf  );

	yyfree( (void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state(  );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack();

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state(  );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state(  );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack (void)
{
	yy_size_t num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		(yy_buffer_stack) = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));

		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (const char * yystr )
{
    
	return yy_scan_bytes( yystr, (int) strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (const char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) (_yybytes_len + 2);
	buf = (char *) yyalloc( n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yynoreturn yy_fatal_error (const char* msg )
{
			fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int yyget_lineno  (void)
{
    
    return yylineno;
}

/** Get the input stream.
 * 
 */
FILE *yyget_in  (void)
{
        return yyin;
}

/** Get the output stream.
 * 
 */
FILE *yyget_out  (void)
{
        return yyout;
}

/** Get the length of the current token.
 * 
 */
int yyget_leng  (void)
{
        return yyleng;
}

/** Get the current token.
 * 
 */

char *yyget_text  (void)
{
        return yytext;
}

/** Set the current line number.
 * @param _line_number line number
 * 
 */
void yyset_lineno (int  _line_number )
{
    
    yylineno = _line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * 
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  _in_str )
{
        yyin = _in_str ;
}

void yyset_out (FILE *  _out_str )
{
        yyout = _out_str ;
}

int yyget_debug  (void)
{
        return yy_flex_debug;
}

void yyset_debug (int  _bdebug )
{
        yy_flex_debug = _bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = NULL;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = NULL;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer( YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, const char * s2, int n )
{
		
	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (const char * s )
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size )
{
			return malloc(size);
}

void *yyrealloc  (void * ptr, yy_size_t  size )
{
		
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void yyfree (void * ptr )
{
			free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 163 "rustlexer.lpp"


